/* This file is part of tagr.
   Copyright (C) 2009 Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA. */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>
#include <tagr.h>

int
create_hierarchy (char *dir, int perm)
{
  int rc;
  struct stat st;
  char *p;

  if (stat (dir, &st) == 0)
    {
      if (!S_ISDIR (st.st_mode))
	{
	  logmsg (L_ERR, _("component %s is not a directory"), dir);
	  return 1;
	}
      return 0;
    }
  else if (errno != ENOENT)
    {
      logmsg (L_ERR, _("cannot stat file %s: %s"), dir, strerror (errno));
      return 1;
    }

  p = strrchr (dir, '/');
  if (p)
    *p = 0;
  rc = create_hierarchy (dir, perm);
  if (rc == 0)
    {
      if (p)
	*p = '/';
      if (mkdir (dir, perm))
	{
	  logmsg (L_ERR, _("cannot create directory %s: %s"),
		  dir, strerror (errno));
	  rc = 1;
	}
    }
  return rc;
}

static int
do_update_output (SD *host,
		  queue_t *queue, const struct avg_acc *avg,
		  time_t timestamp,
		  size_t step, size_t count,
		  char *dirname, char *img_suffix)
{
  char *fname;
  FILE *fp;
  int rc;
  
  if (create_hierarchy (dirname, 0755))
    {
      logmsg (L_ERR, _("cannot create directory %s"),
	      dirname);
      return 1;
    }

  fname = mkfilename (dirname, host->name, img_suffix);
  fp = fopen (fname, "w");
  if (!fp)
    {
      logmsg (L_ERR, _("cannot create file %s: %s"), fname, strerror (errno));
      rc = 1;
    }
  else
    {
      rc = draw_graph (fp, queue, avg, timestamp, step, count,
		       host->max, 0);
      fclose (fp);
    }
  free (fname);
  return rc;
}

int
update_output (SD *host, struct traffic_record *tr, time_t timestamp)
{
  char *dirname = mkfilename (basedir, host->dir, NULL);
  int rc;
  
  rc = do_update_output (host, &tr->day_hist, NULL,
			 DAY_SAMPLE, DAY_COUNT, timestamp,
			 dirname, "-day.png");
  rc += do_update_output (host, &tr->week_hist, &tr->week_avg,
			  WEEK_SAMPLE, WEEK_COUNT, timestamp,
			  dirname, "-week.png");
  rc += do_update_output (host, &tr->month_hist, &tr->month_avg,
			  MONTH_SAMPLE, MONTH_COUNT, timestamp,
			  dirname, "-month.png");
  rc += do_update_output (host, &tr->year_hist, &tr->year_avg,
			  YEAR_SAMPLE, YEAR_COUNT, timestamp,
			  dirname, "-year.png");
  free (dirname);
  return rc;
}
